import os
import sys
import math
import string
import random
import Numeric
from MLab import eye
from LinearAlgebra import singular_value_decomposition, determinant
import Bio.PDB
import Pmeso_Weights
from MesoTypes import meso_types
from MesoDict import MESO_VALUES
from AlfTau2Meso import RC_DICT
from ContactEnergies import CONTACT_E



DEGRAD = math.pi/180.0
RADDEG = 180.0/math.pi

# for fccd
S=eye(3,3, typecode='d')
S[2,2]=-1

def fccd(moving, fixed, threshold=0.01, maxit=10000):
    if len(moving)<6:
        raise "Moving too short"
    if len(fixed)!=3:
        raise "Fixed should have length 3"
    lng=len(moving)
    # Coordinates along COLUMNS
    moving_coords=Numeric.zeros((3, 3), 'd')
    fixed_coords=Numeric.zeros((3, 3), 'd')
    it=0
    while 1:
        if it==maxit:
            # Stop - max iterations reached
            return  rmsd, it, moving
        for i in range(2, lng-2):
            it=it+1
            center=moving[i]
            # move to pivot origin
            for j in range(0, 3):
                index=-(3-j)
                v=moving[index]-center
                moving_coords[:,j]=v.get_array()
            for j in range(0, 3):
                v=fixed[j]-center
                fixed_coords[:,j]=v.get_array()
            # Do SVD
            a=Numeric.matrixmultiply(fixed_coords, Numeric.transpose(moving_coords))
            u, d, vt=singular_value_decomposition(a)
            # Check reflection
            if (determinant(u)*determinant(vt))<0:
                u=Numeric.matrixmultiply(u, S)
            # Calculate rotation
            rot_matrix=Numeric.matrixmultiply(u, vt)
            assert(determinant(rot_matrix)>0)
            # Apply rotation
            for j in range(i+1, len(moving)):
                v=moving[j]-center
                v=v.left_multiply(rot_matrix)
                v=v+center
                moving[j]=v
            # Calculate RMSD
            rmsd=0.0
            for j in range(1, 4):
                m=moving[-j]
                f=fixed[-j]
                n=(m-f).norm()
                rmsd+=n*n
            rmsd=math.sqrt(rmsd/3.0)
#           print "Iteration ", it, "RMSD ", rmsd
            if rmsd<threshold:
                # stop - RMSD threshold reached
                return rmsd, it, moving

def make_vdw_list(atomlist):
    RMIN2SIG = 1.0/(2.0**(1.0/6.0))

    vlist = []
    for i in xrange(0,len(atomlist)-1):
        a = atomlist[i]
        if a.name == 'HB': continue
        arad = a.d4
        asigma = arad * RMIN2SIG
        if atomlist[i].name == 'CA':
            start = (i + 4)
        elif atomlist[i].name == 'CB':
            start = (i + 3)
        for j in xrange(start,  len(atomlist)):
            b = atomlist[j]
            if b.name == 'HB': continue
            brad = b.d4
            bsigma = brad * RMIN2SIG

            # for hydrophobics and charged - they tend to orient rotamers toward each other
            # and this can't happen with large spherical vdw radius - so adjust for these side chains
            #
            # first the hydrophobics 
            if (a.rnam == 'PHE' or a.rnam == 'TYR' or a.rnam == 'TRP' or a.rnam == 'LEU' or a.rnam == 'ILE') \
               and (b.rnam == 'PHE' or b.rnam == 'TYR' or b.rnam == 'TRP' or b.rnam == 'LEU' or a.rnam == 'ILE') \
               and a.name == 'CB' and b.name == 'CB':
                asigma = asigma * 0.5
                bsigma = bsigma * 0.5

            # charged one direction
            if (a.rnam == 'GLU' or a.rnam == 'ASP') \
               and (b.rnam == 'LYS' or b.rnam == 'ARG') \
               and a.name == 'CB' and b.name == 'CB':
                asigma = asigma * 0.5
                bsigma = bsigma * 0.5

            # charged other direction
            if (a.rnam == 'LYS' or a.rnam == 'ARG') \
               and (b.rnam == 'GLU' or b.rnam == 'ASP') \
               and a.name == 'CB' and b.name == 'CB':
                asigma = asigma * 0.5
                bsigma = bsigma * 0.5

            vlist.append((i, j, (asigma + bsigma)))
#           print a.rnam, a.name, a.d4, asigma, b.rnam, b.name, b.d4, bsigma
    return vlist

def make_hbond_list(atomlist):
    hb_atm_indices = get_HB_indx(atomlist)
#   print atomlist[hb_atm_indices[1]].name, atomlist[hb_atm_indices[1]].rnam,atomlist[hb_atm_indices[1]].p1
    hblist = []
    for i in xrange(len(hb_atm_indices)-6):

        # helix forming hbond score
        enehb = 1.0
        for j in xrange(i+3,i+6):
            hblist.append((atomlist[hb_atm_indices[i]],atomlist[hb_atm_indices[j]],enehb))

        # long range hbond score
        enehb = 1.0
        for j in xrange(i+6,len(hb_atm_indices)):
            hblist.append((atomlist[hb_atm_indices[i]],atomlist[hb_atm_indices[j]],enehb))

    return hblist
        

def make_contact_list(atomlist,cntct_atm_indices):
    is_contact_pair = CONTACT_E.has_key
    clist = []
    for i in range(len(cntct_atm_indices)-3):
        for j in range(i+3,len(cntct_atm_indices)):
            iname = atomlist[cntct_atm_indices[i]].rnam
            jname = atomlist[cntct_atm_indices[j]].rnam
            if is_contact_pair((iname, jname)):
                econ = CONTACT_E[(iname, jname)] 
#               print iname, jname, atomlist[cntct_atm_indices[i]].d4, atomlist[cntct_atm_indices[j]].d4,econ
                clist.append((cntct_atm_indices[i],cntct_atm_indices[j], \
                             (atomlist[cntct_atm_indices[i]].d4 +  atomlist[cntct_atm_indices[j]].d4),econ))

    return clist


def angle(a1, a2, a3, acos=math.acos, sqrt=math.sqrt):
    """Angle in degrees between 3 atoms"""
    x1 = a1.x - a2.x
    y1 = a1.y - a2.y
    z1 = a1.z - a2.z

    x2 = a3.x - a2.x
    y2 = a3.y - a2.y
    z2 = a3.z - a2.z

    try:
        ang = (x1*x2 + y1*y2 + z1*z2)/sqrt((x1**2.0 + y1**2.0 + z1**2.0) *
                                           (x2**2.0 + y2**2.0 + z2**2.0))
    except ZeroDivisionError:
        return -999.999

    try:
        return acos(ang)*RADDEG
    except ValueError:
        if ang < -1.0:
            return 180.0
        else:
            return 0.0



def distance(a, b):
    """Euclidean distance between two atoms"""
    return math.sqrt((a.x - b.x)**2.0 + (a.y - b.y)**2.0 + (a.z - b.z)**2.0)

def close(a, b,cscale=1.0, dist=None):
    """check if two atoms are within a specified distance"""
    if dist is None:
        r1 = a.d4 * cscale
        r2 = b.d4 * cscale
        if r1 < 1.0e-6 or r2 < 1.0e-6: return 0
        dist = r1 + r2
    d2 = dist * dist
    dx = (a.x - b.x)**2.0
    if dx >= d2: return 0
    dy = (a.y - b.y)**2.0
    if dy >= d2: return 0
    dz = (a.z - b.z)**2.0
    if dz >= d2: return 0
    
    if dx + dy + dz > d2:
        return 0
    else:
        return 1

def soft_debump_score(atomlist, vlist,cscale=0.6,dbscale=1.0):
    #
    # V = Eps,i,j[(Sigma,i,j/ri,j)**12]
    #
    # where Sigma = (Rmin/2)**(1/6)
    # and Rmin/2 is the vdw radius
    #
    # epsilon: kcal/mole, Eps,i,j = sqrt(eps,i * eps,j)
    # but we will use esp = 0.1 for all atom types
    #
    # Rmin/2: A, Rmin,i,j = Rmin/2,i + Rmin/2,j
    # where Rmin/2 is the atom radius
    #
    # Sigma is precalculated and is in vlist (see construct.py)
    if dbscale < 0.00001:
        return 0.0
    eps = 1.0
    Potential = 0.0
    for i,j,sigma in vlist:
        a = atomlist[i]
        b = atomlist[j]
        sigma = sigma * cscale
        if close(a, b, cscale=cscale):
            dist = distance(a, b)
            V = eps*((sigma/dist)**12)
#           if V > 10.0:
#               print a.indx,a.name,a.rnam,a.rnum,a.d4,a.x,b.indx,b.name,b.rnam,b.rnum,b.d4,b.x,sigma, dist, V
        else:
            V = 0.0
        Potential = Potential + V
    Potential = Potential * dbscale
    return Potential

def contact_score(atomlist, clist, cnscale=1.0):
#      clist.append((cntct_atm_indices[i],cntct_atm_indices[j], \
#       (atomlist[cntct_atm_indices[i]].d4 +  atomlist[cntct_atm_indices[j]].d4),econ))
    if cnscale < 0.00001:
        return 0.0
    shift = 1.0/3.0
    econ = 0.0
    for i,j,rad_sum,E in clist:
        a = atomlist[i]
        b = atomlist[j]
        dmax = rad_sum + 3.0
        dmin = rad_sum - 1.0     #represents overlap
        if close(a,b,cscale=1.0,dist=dmax):
            d12 = distance(a,b)
            if d12 < dmin:
                localE = E
                econ = econ + E            
            else:
                localE = E*(dmax - d12)*shift
                econ = econ + E*(dmax - d12)*shift
#           print ' close',a.indx,a.name,a.rnam,a.rnum,b.indx,b.name,b.rnam,b.rnum,rad_sum,localE
    econ = econ * cnscale
    return -econ

def hbond_score(atomlist, hblist,hbscale=1.0):
    if hbscale < 0.00001:
        return 0.0
    ehb = 0.0
    for a, b, ene in hblist:
#       print a.name, a.rnam, a.p1
        dis = distance(a,b)
        if dis > 4.6 and dis < 5.1:
            ang = angle(atomlist[a.p1], a, b)
            if 70.0 < ang < 100.0:
#               print ' close',a.indx,a.name,a.rnam,a.rnum,ang, b.indx,b.name,b.rnam,b.rnum, dis
                ehb += ene

    return -ehb * hbscale

def res_rc(alf, tau):

    ialf = int(round(alf/10.0 )) * 10
    
    # special case of tau equals exactly 180
    if int(tau) == 180:
        tau = 179.0
    elif int(tau) == -180:
        tau = -179.0

    itau = -15 + int(round((tau + 15)/30.0)) * 30
        
    try:
        return RC_DICT[(ialf,itau)]
    except:
        return 'Xx' 

def load_pmeso_weights(file):
    from string import split, atof
    data = map(split, open(file, 'r').readlines())
    pmeso_weights = []
    for i in range(len(data)):
        prob = [] 
        for j in range(1,73):
            prob.append(atof(data[i][j]))
        prob.insert(0,data[i][0])
        pmeso_weights.append(prob)

    return pmeso_weights

def load_resi_pmeso_weights(file):
    from string import split, atof
    data = map(split, open(file, 'r').readlines())
    resi_pmeso_weights = []
    for i in range(len(data)):
        prob = [] 
        for j in range(72):
            prob.append(atof(data[i][j]))
        resi_pmeso_weights.append(prob)
    
    return resi_pmeso_weights



def meso_state_score(atomlist, ca_atm_indices, pmeso_weights, msscale=1.0):
    if msscale < 0.00001:
        return 0.0
    msscale = msscale * 100.0
    Potential = 0.0
#   pmeso_weights= Pmeso_Weights.pmeso_list()

    for i in range(2,len(ca_atm_indices)-1):
        alf = atomlist[ca_atm_indices[i]].d2
        tau = atomlist[ca_atm_indices[i]].d3
        res =  atomlist[ca_atm_indices[i]].rnam
        res = string.lower(res)
        meso = res_rc(alf,tau)
        if meso == 'Xx':
            continue
        ms_type = meso_types[meso]
        for i in range(len(pmeso_weights)):
            if pmeso_weights[i][0] == res:
                # add +1 to ms_type here because in pmeso_weights the first
                # column is res. name and all type are shifted up one index
                score = float(pmeso_weights[i][ms_type + 1])
                Potential = Potential + score

    # normalize to length of chain (exceptin ends)
    Potential = (Potential * msscale) / (len(ca_atm_indices)-3)
    return -Potential

def choose_meso(res, pmeso_weights):
    for i in range(len(pmeso_weights)):
#       print pmeso_weights[i][0]
        if pmeso_weights[i][0] == res:
            row = pmeso_weights[i][1:]
            sum = 0
            for i in row:
                sum = sum + i
            r = random.uniform(0.0001,sum)
            pos = 0
            rsum = 0
            for i in row:
                rsum = rsum + i
                if r <= rsum:
                    for key, value in meso_types.items():
                        if value == pos:
                            meso = key
                            return meso
                pos += 1


def setconf_pivot(atomlist, res_atm_indx,pmeso_weights):
    meso = choose_meso(string.lower(atomlist[res_atm_indx].rnam),pmeso_weights)
    a,t = MESO_VALUES[meso]
    alf = apply(random.uniform, a)
    tau = apply(random.uniform, t)
    atomlist[res_atm_indx].d2 = alf
    atomlist[res_atm_indx].d3 = tau

def choose_meso_resi_pmeso(resnum, resi_pmeso_weights):
#   print resnum, len(resi_pmeso_weights)
    row = resi_pmeso_weights[resnum-1][0:]
    sum = 0
    for i in row:
        sum = sum + i
    r = random.uniform(0.0001,sum)
    pos = 0
    rsum = 0
    for i in row:
        rsum = rsum + i
        if r <= rsum:
            for key, value in meso_types.items():
                if value == pos:
                    meso = key
                    return meso
        pos += 1

def setconf_resi_pmeso(atomlist,res_atm_indx, resnum,resi_pmeso_weights):
    meso = choose_meso_resi_pmeso(resnum,resi_pmeso_weights)
    a,t = MESO_VALUES[meso]
    alf = apply(random.uniform, a)
    tau = apply(random.uniform, t)
    atomlist[res_atm_indx].d2 = alf
    atomlist[res_atm_indx].d3 = tau


def setconf_local(atomlist, ca_atm_indices, res):
    """
    Local move of 4 residues.
    Needs 10 residues to works with:
        first 3 are static, middle 4 change, and last 3 move en bloc to rmsd
        with target of "fixed" 3
    """


    # get atom indices of CA's for first and last residues in loop
    first_atm_indx = ca_atm_indices[res - 5]
    last_atm_indx = ca_atm_indices[res + 4]

#   print res, first_atm_indx, last_atm_indx

    
    # make list of vectors of x,y,z coords for 10 residue segment
    moving = []
    for i in range(len(ca_atm_indices)):
        if ca_atm_indices[i] >= first_atm_indx and ca_atm_indices[i] <= last_atm_indx:
            v = Bio.PDB.Vector(atomlist[ca_atm_indices[i]].x,
                               atomlist[ca_atm_indices[i]].y,
                               atomlist[ca_atm_indices[i]].z)
            moving.append(v)


    # save the coords of last 3 residues as target
    fixed = []
    for i in range(-3,0):
        fixed.append(moving[i])

#   for i in range(len(fixed)):
#       print fixed[i]

    # make list of vectors of segment where the 3 thru 10'th are normalized so
    # they can be rot-translated and then used to rebuild the chain
    altered = [moving[0]]
    altered.append(moving[1])
    altered.append(moving[2])
    for i in range(3, len(moving)):
        v = moving[i] - moving[i-1]
        v.normalize()
        altered.append(v)


    # set some random parameters for the rot-trans
    rotation_axis=Bio.PDB.Vector(Numeric.array((random.random(),random.random(),random.random()),'d'))
#   angle=0.1+0.1*math.pi*random.random()
    angle=0.001 * math.pi*random.random()
    m=Bio.PDB.rotaxis(angle, rotation_axis)
    t=Bio.PDB.Vector(-0.02*Numeric.array((random.random(),random.random(),random.random()),'d'))


    # apply rot-trans to normalized vectors of resids 3, 4, 5, 6 (0,1,2 and 7,8,9 stay)
    for i in range(3,7):
        altered[i] = altered[i]-t
        altered[i].left_multiply(m)
        altered[i].normalize()

    # rebuild the coords for the chain
    k = altered[2]
    for i in range(3,len(altered)):
        altered[i] = k + altered[i]**3.8
        k = altered[i]


    # find the fit
    rmsd, it, new=fccd(altered, fixed)

    # update xyz coords for residue segment
    # here first 3 should be unchanged
    # last 3 as close as RMSD allowed
    # central 4 have new conformation
    # so we start with 4th residue in segment
    indx = 0
    for i in range(res-5, res+5):
#       print atomlist[ca_atm_indices[i]].x,new[indx][0],atomlist[ca_atm_indices[i]].d2

        atomlist[ca_atm_indices[i]].x = new[indx][0]
        atomlist[ca_atm_indices[i]].y = new[indx][1]
        atomlist[ca_atm_indices[i]].z = new[indx][2]
        indx += 1

    # update alpha and tau for CA atoms
    # essentially do an xtoz
    got_move = 1
    for i in range(res-5, res+5):
#       print atomlist[ca_atm_indices[i]].x
        v1 = Bio.PDB.Vector(atomlist[ca_atm_indices[i-2]].x,
                            atomlist[ca_atm_indices[i-2]].y,
                            atomlist[ca_atm_indices[i-2]].z)
        v2 = Bio.PDB.Vector(atomlist[ca_atm_indices[i-1]].x,
                            atomlist[ca_atm_indices[i-1]].y,
                            atomlist[ca_atm_indices[i-1]].z)
        v3 = Bio.PDB.Vector(atomlist[ca_atm_indices[i]].x,
                            atomlist[ca_atm_indices[i]].y,
                            atomlist[ca_atm_indices[i]].z)
        v4 = Bio.PDB.Vector(atomlist[ca_atm_indices[i+1]].x,
                            atomlist[ca_atm_indices[i+1]].y,
                            atomlist[ca_atm_indices[i+1]].z)
        alf = Bio.PDB.calc_angle(v2,v3,v4) * RADDEG
        if alf < 85.0 or alf > 145.0:
            got_move = 0
        tau = Bio.PDB.calc_dihedral(v1,v2,v3,v4) *RADDEG
#       print atomlist[ca_atm_indices[i]].d2, alf
        atomlist[ca_atm_indices[i]].d2 = alf
        atomlist[ca_atm_indices[i]].d3 = tau

    return got_move


def int2vec(v1, dis, v2, ang, v3, tors, sin=math.sin,\
            cos=math.cos, sqrt=math.sqrt):
    """int2vec(v1, dis, v2, ang, v3, tors)
        Compute coordinates for vector v, given its 'dis'tance
        from vector v1, 'ang'le v-v1-v2 and 'tors'ion v-v1-v2-v3.'
        Both ang and tors should be in degrees.
    """

    ang = ang*DEGRAD
    sina = sin(ang)
    cosa = cos(ang)

    tors = (tors*DEGRAD)
    sint = sina * sin(tors)
    cost = sina * cos(tors)

    x1, y1, z1 = v1
    x2, y2, z2 = v2
    x3, y3, z3 = v3

    u1x = x2 - x3
    u1y = y2 - y3
    u1z = z2 - z3
    d = 1.0 / sqrt(u1x*u1x + u1y*u1y + u1z*u1z)
    u1x = u1x*d
    u1y = u1y*d
    u1z = u1z*d

    u2x = x1 - x2
    u2y = y1 - y2
    u2z = z1 - z2;
    d = 1.0 / sqrt(u2x*u2x + u2y*u2y + u2z*u2z)
    u2x = u2x*d
    u2y = u2y*d
    u2z = u2z*d

    cosine = u1x*u2x + u1y*u2y + u1z*u2z

    if (abs(cosine) < 1.0):
        sine = 1.0/sqrt(1.0 - cosine*cosine)
    else:
        sine = 1.0/sqrt(cosine*cosine - 1.0)

    u3x = sine * (u1y*u2z - u1z*u2y)
    u3y = sine * (u1z*u2x - u1x*u2z)
    u3z = sine * (u1x*u2y - u1y*u2x)

    u4x = cost * (u3y*u2z - u3z*u2y)
    u4y = cost * (u3z*u2x - u3x*u2z)
    u4z = cost * (u3x*u2y - u3y*u2x)

    return [x1 + dis*(-u2x*cosa + u4x + u3x*sint),
            y1 + dis*(-u2y*cosa + u4y + u3y*sint),
            z1 + dis*(-u2z*cosa + u4z + u3z*sint)]


def sc2vec(v1, v2, v3, dis=3.0):
    theta = 37.6 * DEGRAD

    # expand coords from passed vectors
    x1, y1, z1 = v1     # r(i-1), parent of next CA
    x2, y2, z2 = v2     # r(i), CA bonded to CB
    x3, y3, z3 = v3     # r(i+1), child of previous CA

#   print x1, y1, z1
#   print x2, y2, z2
#   print x3, y3, z3

    #          (ri - ri-1) + (ri - ri+1)
    #  X  =   ---------------------------
    #         |(ri - ri-1) + (ri - ri+1)|

    x1x = x2 - x1
    x1y = y2 - y1
    x1z = z2 - z1

    x2x = x2 - x3
    x2y = y2 - y3
    x2z = z2 - z3

    xnx = x1x + x2x
    xny = x1y + x2x
    xnz = x1x + x2z
 
#   print xnx, xny, xnz

    dx = math.sqrt(xnx*xnx)
    if dx == 0.0:
        xnx = 0.0
    else:
        xnx = xnx/dx * DEGRAD

    dy = math.sqrt(xny*xny)
    if dy == 0.0:
        xny = 0.0
    else:
        xny = xny/dy * DEGRAD

    dz = math.sqrt(xnz*xnz)
    if dz == 0.0:
        xnz = 0.0
    else:
        xnz = xnz/dz * DEGRAD


    #          (ri - ri-1) * (ri - ri+1)
    #  Y  =   ---------------------------
    #         |(ri - ri-1) * (ri - ri+1)|


    y1x = x2 - x1
    y1y = y2 - y1
    y1z = z2 - z1

    y2x = x2 - x3
    y2y = y2 - y3
    y2z = z2 - z3

    ynx = x1x * x2x
    yny = x1y * x2x
    ynz = x1x * x2z

#   print ynx, yny, ynz

    dx = math.sqrt(ynx*ynx)
    if dx == 0.0:
        ynx = 0.0
    else:
        ynx = ynx/dx * DEGRAD

    dy = math.sqrt(yny*yny)
    if dy == 0.0:
        yny = 0.0
    else:
        yny = yny/dy * DEGRAD

    dz = math.sqrt(ynz*ynz)
    if dz == 0.0:
        ynz = 0.0
    else:
        ynz = ynz/dz * DEGRAD

    # R = dis * cos(theta * X) + dis * sin(theta * Y)

    return [(dis * math.cos(theta * xnx)) + (dis * math.sin(theta * ynx)),
            (dis * math.cos(theta * xny)) + (dis * math.sin(theta * yny)),
            (dis * math.cos(theta * xnz)) + (dis * math.sin(theta * ynz))]

def ztox(atomlist, ca_atm_indices,cb_atm_indices):

    # first do backbone
    for i in xrange(3, len(ca_atm_indices)):
        a = atomlist[ca_atm_indices[i]]
        p1 = a.p1  
        p2 = atomlist[p1].p1
        p3 = atomlist[p2].p1
        v1 = []
        v1.append(atomlist[p1].x)
        v1.append(atomlist[p1].y)
        v1.append(atomlist[p1].z)
        v2 = []
        v2.append(atomlist[p2].x)
        v2.append(atomlist[p2].y)
        v2.append(atomlist[p2].z)
        v3 = []
        v3.append(atomlist[p3].x)
        v3.append(atomlist[p3].y)
        v3.append(atomlist[p3].z)
        dist = a.d1
        alf = atomlist[p1].d2
        tau = atomlist[p1].d3

        xyz = int2vec(v1, dist, v2, alf, v3, tau)
        a.x = xyz[0]
        a.y = xyz[1]
        a.z = xyz[2]

    # now build CB atoms on new backbone
    cb_atm_indices = get_CB_indx(atomlist)
    for i in xrange(len(cb_atm_indices)):
        a = atomlist[cb_atm_indices[i]]

        p1 =  a.p1
        p2 = atomlist[p1].p1
        p3 = p1 + 3
        v1 = []
        v1.append(atomlist[p1].x)
        v1.append(atomlist[p1].y)
        v1.append(atomlist[p1].z)
        v2 = []
        v2.append(atomlist[p2].x)
        v2.append(atomlist[p2].y)
        v2.append(atomlist[p2].z)
        v3 = []
        v3.append(atomlist[p3].x)
        v3.append(atomlist[p3].y)
        v3.append(atomlist[p3].z)
        dis = a.d1
        alf = atomlist[p1].d2

#       print a.indx,a.name,a.rnam,a.rnum,a.d2,a.d3

        ang = (360.0 - alf)/2.0
        tor = a.d3
        xyz = int2vec(v1, dis, v2, ang, v3, tor)
        a.x = xyz[0]
        a.y = xyz[1]
        a.z = xyz[2]

    # Build backbone HB's
    for i in xrange(2, len(ca_atm_indices)):
        postCA = atomlist[ca_atm_indices[i]]
        preCA = atomlist[postCA.p1] 
        HB = atomlist[ca_atm_indices[i]-1]
        HB.x = (postCA.x + preCA.x) / 2.0
        HB.y = (postCA.y + preCA.y) / 2.0
        HB.z = (postCA.z + preCA.z) / 2.0

def write_pdb(prot_name, atomlist,file,ecur):
    pdbfmt = 'ATOM  %5d  %-3s %-3s  %4d    %8.3f%8.3f%8.3f  0.00  0.00\n'
    file.write('COMPND    %s %8.3f\n' % (prot_name,ecur))
    for i in range(len(atomlist)):
        a = atomlist[i]
#       if a.name == 'CB': continue
#       if a.name == 'HB': continue
        file.write(pdbfmt % (a.indx,a.name,a.rnam,int(a.rnum),a.x,a.y,a.z))
    file.write('TER\n')
    file.close()

def write_traj(atomlist,file,cyc,ecur):
    if ecur == None:
        ecur = 0.0
    pdbfmt = 'ATOM  %5d  %-3s %-3s  %4d    %8.3f%8.3f%8.3f  0.00  0.00\n'
    file.write('MODEL Cycle: %9d Energy: %8.3f\n' % (cyc,ecur))
    for i in range(len(atomlist)):
        a = atomlist[i]
        file.write(pdbfmt % (a.indx,a.name,a.rnam,int(a.rnum),a.x,a.y,a.z))
    file.write('TER\n')
    file.write('ENDMDL\n')


def commit_internal_coords(atomlist, al_copy):
    for i in range(len(atomlist)):
        al_copy[i].d2 = atomlist[i].d2
        al_copy[i].d3 = atomlist[i].d3
        al_copy[i].x = atomlist[i].x
        al_copy[i].y = atomlist[i].y
        al_copy[i].z = atomlist[i].z

def restore_internal_coords(atomlist, al_copy):
    for i in range(len(atomlist)):
        atomlist[i].d2 = al_copy[i].d2
        atomlist[i].d3 = al_copy[i].d3
        atomlist[i].x = al_copy[i].x
        atomlist[i].y = al_copy[i].y
        atomlist[i].z = al_copy[i].z

def HullVolume(coords, hlscale = 1.0):
    # coords are in a list such as,
    # coords.append((float(line[0]), float(line[1]), float(line[2])))
    # etc

    if hlscale < 0.00001:
        return 0.0
    qhull_path = "/usr/local/bin/"
    dim = 3
    num_points = len(coords)
    instring = " %d\n " % (dim )
    instring += " %d\n " % (num_points )
    for i in range(len(coords)):
        instring += ' %f %f %f\n' % (coords[i][0],coords[i][1],
                                   coords[i][2])

    instring += '\n' 
#   print instring
    input, output =  os.popen4(os.path.join(qhull_path, "qconvex") + " FA")
    input.write(instring)
    input.close()

    data = output.read()
    #print data[data.find("Total volume:") + 14 :]
    area_start = data.find("area:")
    area_end = data.find("Total volume:")
    a = float(data[(area_start + 5) : area_end] )
    vol_start = data.find("volume:")
    if vol_start == -1 :
        print data
        return 0
    v = float(data[vol_start + 7 :] )
    ratio = (a / v) * hlscale

    return ratio

def get_coords(atomlist):
    coords = []
    for i in range(len(atomlist)):
        coords.append((atomlist[i].x, atomlist[i].y, atomlist[i].z))
    return coords

def get_CA_indx(atomlist):
    ca_atm_indices = []
    for i in range(len(atomlist)):
        if atomlist[i].name == 'CA':
            ca_atm_indices.append(atomlist[i].indx)

    return ca_atm_indices

def get_CB_indx(atomlist):
    cb_atm_indices = []
    for i in range(len(atomlist)):
        if atomlist[i].name == 'CB':
            cb_atm_indices.append(atomlist[i].indx)

    return cb_atm_indices

def get_HB_indx(atomlist):
    hb_atm_indices = []
    for i in range(len(atomlist)):
        if atomlist[i].name == 'HB':
            hb_atm_indices.append(atomlist[i].indx)

    return hb_atm_indices

def get_CNTCT_indx(atomlist):
    cntct_atm_indices = []
    for i in range(len(atomlist)):
        if atomlist[i].rnam == 'GLY' and atomlist[i].name == 'CA':
            cntct_atm_indices.append(atomlist[i].indx)
        if atomlist[i].name == 'CB':
            cntct_atm_indices.append(atomlist[i].indx)

    return cntct_atm_indices


def radius_of_gyration(atomlist):
    rg = x = y = z = 0.0
    for i in range(len(atomlist)):
        x = x + atomlist[i].x
        y = y + atomlist[i].y
        z = z + atomlist[i].z
    n = 1.0/len(atomlist)
    x = x*n
    y = y*n
    z = z*n
    for i in range(len(atomlist)):
        rg = rg + (atomlist[i].x - x)**2.0 + (atomlist[i].y - y)**2.0 + (atomlist[i].z - z)**2.0
    rg = math.sqrt(rg * n) 
    return rg

def rg_confinement_score(atomlist,Rg_predict,rgscale=1.0):
#   Rg_predict =  2.5 * (math.pow(NUMRES, 0.34))
    rgscale = rgscale * 0.1
    Rg = radius_of_gyration(atomlist)
    if Rg > Rg_predict:
        return ((Rg - Rg_predict)**2.0) * rgscale
    else:
        return 0.0

