"""
Makes a data list of CA distance vectors for input to Pycluster.
Requires of list of PDB files as input.

Output is of the form,
2 3 6 
YAL001  1       1.3     2.4     5.8     2.4
YAL002  0.9     0.8     0.7     0.5     0.2
YAL003  0.8     2.1     4.2     10.1    10.1
YAL004  1.1     1.3     0.8     1.0     0.4
YAL005  1.2     1       1.1     4.5     8.3
YAL006  1.0     1.1     0.9     1.2     0.6

where first row is: 
        number of columns of phi/psi data
        number of colmuns of CA distance data
        number of rows
   
then each row is label + data values.

"""

import sys
import os
from redux_2_1 import PDBio_PDB
import math
from redux_2_1 import Functions

def run(file_list, output):
    frm = -1
    flist = open(file_list, 'r').readlines()
    for file in flist:
        file = file.strip()

        # basename will be the label for each row of data
        basename = os.path.splitext(file)[0]

        prot_name, atomlist = PDBio_PDB.protein_from_pdb(file)
        ca_atm_indices = Functions.get_CA_indx(atomlist)
        end = 0
        if atomlist[0].rnam == 'END': end = 1
        ncas = len(ca_atm_indices) - (2 * end)

        # make the CA distance matrix (abbreviated)
        camtrx = []
        for i in range((end + 1),ncas-7):
            for j in range(i+6,(ncas - end - 1)):
                dist = (Functions.distance(atomlist[i],atomlist[j])) * 100.0/ ncas
                camtrx.append(dist)

        # if this is first cycle, print
        #  number of columns of dummy data (to be consistent with phi/psi data)
        #  number of columns of CA distance data
        #  number of rows
        if frm == -1:
            output.write('%d %d %d\n' % (2, len(camtrx), len(flist)))
        frm += 1

        # print row label
        output.write('%-15s ' % basename)

        # print dummy phi/psi data
        output.write(' %8.3f %8.3f' %  (1.0, 1.0))

        # print CA distance data
        for i in range(len(camtrx)):
            output.write(' %8.3f ' %  (camtrx[i]))

        # end row
        output.write('\n')



if __name__ == "__main__":
    import sys

    try:
        file_list = sys.argv[1]
    except:
        sys.exit()

    output = open('data_file', 'wb')
    run(file_list, output)

