import sys
import os
import time
import math
import string
import random
import copy
from redux_2_1 import PDBio
from redux_2_1 import Functions
from redux_2_1 import Pmeso_Weights
    
def run_sim(filename):

    # Make the all important atomlist
    prot_name, atomlist = PDBio.protein_from_pdb(filename)

    # Stuff you probably will need
    ca_atm_indices = Functions.get_CA_indx(atomlist)
    cb_atm_indices = Functions.get_CB_indx(atomlist)
    cntct_atm_indices = Functions.get_CNTCT_indx(atomlist)
    vlist = Functions.make_vdw_list(atomlist)
    clist = Functions.make_contact_list(atomlist,cntct_atm_indices)
    hblist = Functions.make_hbond_list(atomlist)
    pmeso_weights= Pmeso_Weights.pmeso_list()
#   pmeso_weights= Functions.load_pmeso_weights('pmeso_weights')
    resi_pmeso_weights= Functions.load_resi_pmeso_weights('resi_pmeso_weights')
    numres = int(atomlist[len(atomlist)-2].rnum)

####### EDIT THESE IF DESIRED #########

    # How large should the confinement cage be (2.8 is normal, 2.5 is small, 3.5 is big)
    # The cage will gradually shrink to the end size
    start_rg_prefac = 3.0
    end_rg_prefac = 2.5
    Rg_predict =  start_rg_prefac * (math.pow(numres, 0.34))


    # Scale the various scoring functions
    # Set to 0.0 to turn off that function 
    #
    # Confinement
    rgscale = 0.1     # 0.1 default
    #
    # Soft Debump
    dbscale = 1.0     # 0.1 default - this is overall score scale
    cscale = 0.6      # 0.6 max - this is atom size scaling
                      # This will gradually increse - see below
    #
    # Residue Specific Contact
    cnscale = 0.0     # 1.0 default
    #
    # Hydrogen bond
    hbscale = 1.0     # 1.0 default
    #
    # Backbone Mesostate Propensity
    msscale = 0.0     # 1.0 default


    # How many cycles: where a cycle is numres attempts to get a successful pivot move?
    ncycles = 500

    # Save a frame every how many cycles of pivot moves:
    save_int = 10

    # How many local move attempts for each successful pivot move?
    l_trials =  0

######## END EDIT ############

    # Open file for writing trajectory
    traj_file = open('traj.pdb', 'w')

    # Store two copies of internal coords
    al_copy = min_atomlist = copy.deepcopy(atomlist)
    Functions.commit_internal_coords(atomlist, al_copy)

    # How many attempts to get a debumped pivot move or local move?
    # (This usually happens within ~5 attempts, but leave it larger for difficult topologies)
#   nattempts = range(numres)
    nattempts = range(10)

    # Reference energy for startng structure
    eref = Functions.rg_confinement_score(atomlist,Rg_predict,rgscale=rgscale) + \
           Functions.contact_score(atomlist, clist, cnscale=cnscale) + \
           Functions.hbond_score(atomlist,hblist, hbscale=hbscale) + \
           Functions.meso_state_score(atomlist, ca_atm_indices,pmeso_weights, msscale=msscale)
    
    # Store ebest for determining minimum energy structure
    ebest = ecur = eref

    # Approximate room temperature
    beta = 2.0
    
    # Start simulation loop
    for cyc in range(ncycles):
        for XXX in range(numres):

            # Choose residue to move
            resnum = random.randint(3,numres)

            # Find the atom index of CA atom for that residue
            for i in range(len(atomlist)):
                if atomlist[i].name == 'CA' and int(atomlist[i].rnum) == resnum:
                    res_atm_indx = i
    
            # Try for a debumped structure
            success = 0
            for at in nattempts:

                # Choose alf, tau based on probability mesostates
#               Functions.setconf_pivot(atomlist, res_atm_indx, pmeso_weights)
                Functions.setconf_resi_pmeso(atomlist, res_atm_indx, resnum, resi_pmeso_weights)
        
                # Build structure with new angle and dihedral
                Functions.ztox(atomlist, ca_atm_indices, cb_atm_indices)
        
                # Test for extreme vdw clash
                # (See "cscale" in Functions.soft_debump_score for vdw scaling,
                # "dbscale" here is scaling of final score value.)
                ebump = Functions.soft_debump_score(atomlist,vlist,cscale=cscale,dbscale=dbscale)
                if ebump < 1.0:
                    success = 1
                    break
                else:
                    # vdw clash, try again
                    Functions.restore_internal_coords(atomlist, al_copy)

            # If failed bump check, try another residue
            if not success:
                Functions.restore_internal_coords(atomlist, al_copy)

            # If passed bump check, do Metropolis
            elif success:
#               print 'passed pivot bump check, on residue ', resnum, 'attempt '

                ecur = Functions.rg_confinement_score(atomlist,Rg_predict,rgscale=rgscale) + \
                       Functions.contact_score(atomlist, clist, cnscale=cnscale) + \
                       Functions.hbond_score(atomlist, hblist,hbscale=hbscale) + \
                       Functions.meso_state_score(atomlist, ca_atm_indices,pmeso_weights, msscale=msscale)
        
                # Apply Metropolis criterion
                try:
                    metropolis = math.exp(beta * (eref - ecur))
                except OverflowError: 
                    print 'Metropolis overflowerror'
                    metropolis = 1.0

                if metropolis >= random.random():
        
#                   print 'pivot SUCCESS '
                    eref = ecur
#                   Functions.write_traj(atomlist, traj_file,cyc, ecur)

#                   last_file = open('last_good.pdb', 'w')
#                   Functions.write_pdb(prot_name, atomlist, last_file, ecur)
#                   last_file.close()
        
                    # Make new angle and dihedral permanent
                    Functions.commit_internal_coords(atomlist, al_copy)
        
                    # If this is a minimum energy conformation, save its atomlist
                    if ecur < ebest:
                        ebest = ecur
                        min_atomlist = copy.deepcopy(atomlist)
                        pdb_file = open('min.pdb', 'w')
                        prot_name = 'Min Conf'
                        Functions.write_pdb(prot_name, min_atomlist, pdb_file, ebest)
                        pdb_file.close()


                    # Now do a local move
                    local_attmp = 0
                    while local_attmp < l_trials:
    
                        # Choose residue to move
                        resnum = random.randint(5,numres-4)

                        # Try for a debumped structure
                        success = 0
                        for local_at in range(5):
            
                            # and try ten times to get solution that has 85 > alf < 145
                            for l_try in range(10):
                                got_local = 0
                                got_local = Functions.setconf_local(atomlist, ca_atm_indices, resnum)
#                               print ' local try ', l_try
                                if got_local:
#                                   print ' got local on attempt: ',l_try
                                    break
                                else:
                                    Functions.restore_internal_coords(atomlist, al_copy)
    
                            if not got_local: 
#                               print ' failed local_move'
                                # int coords restored above
                                break 
                            elif got_local:

                                # Build structure with new angle and dihedral
                                Functions.ztox(atomlist, ca_atm_indices, cb_atm_indices)

                                # Test for extreme vdw clash
                                # (See "cscale" in Functions.soft_debump_score for vdw scaling,
                                # "dbscale" here is scaling of final score value.)
                                ebump = Functions.soft_debump_score(atomlist,vlist,cscale=cscale,dbscale=dbscale)
                                if ebump < 1.0:
                                    success = 1
                                else:
                                    # vdw clash, try again
                                    Functions.restore_internal_coords(atomlist, al_copy)

                        # If failed local move or bump check, try another residue
                        if not success:
#                           print 'failed local move '
                            # this may not be necessary leave in for safety
                            Functions.restore_internal_coords(atomlist, al_copy)

                        # If passed bump check, do Metropolis
                        elif success:
#                           print 'passed local move bump check on attempt '

                            ecur = Functions.rg_confinement_score(atomlist,Rg_predict,rgscale=rgscale) + \
                                   Functions.contact_score(atomlist, clist, cnscale=cnscale) + \
                                   Functions.hbond_score(atomlist, hblist,hbscale=hbscale) + \
                                   Functions.meso_state_score(atomlist, ca_atm_indices,pmeso_weights, msscale=msscale)
       
                            try:
                                metropolis = math.exp(beta * (eref - ecur))
                            except OverflowError:
                                metropolis = 1.0
                            if metropolis >= random.random():
#                               print 'local move SUCCESS local attempt ', local_attmp
#                               Functions.write_traj(atomlist, traj_file,cyc,ecur)

#                               print '          debump =   ', Functions.soft_debump_score(atomlist,vlist,cscale=cscale,dbscale=dbscale)
#                               print '          confine =  ', Functions.rg_confinement_score(atomlist,Rg_predict,rgscale=rgscale) 
#                               print '          contact =  ', Functions.contact_score(atomlist, clist, cnscale=cnscale)
#                               print '          meso =     ', Functions.meso_state_score(atomlist, ca_atm_indices,pmeso_weights, msscale=msscale)
#                               print '          hbond =    ', Functions.hbond_score(atomlist,hblist,hbscale=hbscale)
#                               print ' ' 


#                               last_file = open('last_good.pdb', 'w')
#                               Functions.write_pdb(prot_name, atomlist, last_file, ecur)
#                               last_file.close()

                                eref = ecur
                                Functions.commit_internal_coords(atomlist, al_copy)

                                # If this is a minimum energy conformation, save its atomlist
                                if ecur < ebest:
                                    ebest = ecur
                                    min_atomlist = copy.deepcopy(atomlist)

                            else:
#                               print 'local move FAILURE'
                                Functions.restore_internal_coords(atomlist, al_copy)
                                # this may not be necessary, leave in for safety
                                Functions.ztox(atomlist, ca_atm_indices, cb_atm_indices)
 
                        # counter for local moves
                        local_attmp += 1

                else:
#                   print 'pivot FAILURE', 'ecur = ', ecur, 'eref = ', eref
                    Functions.restore_internal_coords(atomlist, al_copy)
                    # this may not be necessary, leave in for safety
                    Functions.ztox(atomlist, ca_atm_indices, cb_atm_indices)

        # Squeeze it
        if start_rg_prefac > end_rg_prefac:
            start_rg_prefac = start_rg_prefac - 0.1
            Rg_predict =  start_rg_prefac * (math.pow(numres, 0.34))

        # Expand vdw radius
        if cscale < 0.6:
            cscale += 0.01

        # write trajectory every "save" moves
        if not cyc % save_int:
            Functions.write_traj(atomlist, traj_file,cyc, ecur)


    
    # End of main simulation loop

    # Write out the minimum energy conformation
    traj_file.close()
    pdb_file = open('min.pdb', 'w')
    prot_name = 'Final'
    Functions.write_pdb(prot_name, min_atomlist, pdb_file, ebest)
    pdb_file.close()
     
if __name__ == "__main__":
    sys.stdout.write('Simulation starting time: %s\n' % time.ctime(time.time()))
    filename = sys.argv[1]
    file = os.path.basename(filename)
    basedir = os.getcwd()
    curdir = basedir + os.sep + 'sim2'
    os.mkdir(curdir)
    os.link(filename, curdir+os.sep+file)
    os.link('resi_pmeso_weights', curdir+os.sep+'resi_pmeso_weights')
    os.chdir(curdir)
    run_sim(file)
    sys.stdout.write('Simulation Finished at: %s\n' % time.ctime(time.time()))
