"""
Clusters protein structures using a combined string of phi/psi values
and CA distance values.

Input (from Mk_camatrix_filelist.py) is of the form:

2 3 6 
YAL001  1       1.3     2.4     5.8     2.4
YAL002  0.9     0.8     0.7     0.5     0.2
YAL003  0.8     2.1     4.2     10.1    10.1
YAL004  1.1     1.3     0.8     1.0     0.4
YAL005  1.2     1       1.1     4.5     8.3
YAL006  1.0     1.1     0.9     1.2     0.6

where first row is: 
	number of columns of phi/psi data
	number of colmuns of CA distance data
	number of rows
   			
then each row is label + data values.

Standard output is of the form:

0.0329811079091 2
Cluster  1 YAL006
Cluster  1 YAL004
0.0690041626641 3
Cluster  2 YAL006
Cluster  2 YAL004
Cluster  2 YAL002
0.545308997759 4
Cluster  3 YAL006
Cluster  3 YAL004
Cluster  3 YAL002
Cluster  3 YAL001
0.677233429395 2
Cluster  4 YAL003
Cluster  4 YAL005
1.0 6
Cluster  5 YAL003
Cluster  5 YAL005
Cluster  5 YAL006
Cluster  5 YAL004
Cluster  5 YAL002
Cluster  5 YAL001

where the unlabeled entries are the cluster distance and cluster membership
and below each unlabeled entry is a list of members for that cluster.

Also output are two files suitable for TreeView:

out.cdt 

GID     UNIQID  NAME    GWEIGHT 0       1       2       3       4
EWEIGHT                         1       1       1       1       1
GENE0   YAL001  YAL001  1       1.0     1.3     2.4     5.8     2.4
GENE1   YAL002  YAL002  1       0.9     0.8     0.7     0.5     0.2
GENE3   YAL004  YAL004  1       1.1     1.3     0.8     1.0     0.4
GENE5   YAL006  YAL006  1       1.0     1.1     0.9     1.2     0.6
GENE2   YAL003  YAL003  1       0.8     2.1     4.2     10.1    10.1
GENE4   YAL005  YAL005  1       1.2     1.0     1.1     4.5     8.3

out.gtr 

NODE1   GENE5   GENE3   0.967018892091
NODE2   NODE1   GENE1   0.930995837336
NODE3   NODE2   GENE0   0.454691002241
NODE4   GENE2   GENE4   0.322766570605
NODE5   NODE4   NODE3   0.0

which have the same information as the standard output.

Notice that the standard output lists cluster distance whereas
the out.gtr lists cluster similarity.
"""
USAGE = """
python new_cluster.py data_file > out.lis
"""

import Numeric
import sys,string
from Pycluster import treecluster2,writeclusterfiles,treecluster

datafile = open(sys.argv[1],'rb')
lines = datafile.readlines()
numrow = int(string.split(string.strip(lines[0]))[2])

numcol = int(string.split(string.strip(lines[0]))[0]) + \
         int(string.split(string.strip(lines[0]))[1])

num_circular = int(string.split(string.strip(lines[0]))[0])
num_linear = int(string.split(string.strip(lines[0]))[1])

weight_row = []
for i in xrange(num_circular):
    weight_row.append(0.0)
for i in xrange(num_linear):
    weight_row.append(1.0)
eweight = Numeric.zeros(numcol,'d')
for i in xrange(numcol):
    eweight[i] = weight_row[i]
#print eweight
    

expid = []
for i in xrange(numcol):
    expid.append(str(i))
#print expid

labels = []
for i in xrange(1,numrow + 1):
    labels.append(string.split(string.strip(lines[i]))[0])
#print labels

datarray = Numeric.zeros((numrow,numcol),'d')
for i in xrange(1,numrow + 1):
    line = lines[i]
    j = 1
    for j in xrange(1,numcol + 1):
        datarray[i-1][j-1] = float((string.split(string.strip(lines[i]))[j]))

#print datarray

tree, linkdist = treecluster(data=datarray,weight=eweight,method='c',dist='a')
#print tree
#linkdist = (linkdist/max(linkdist))

nlinkdist = (1.0 - linkdist)
#print nlinkdist

node = []
for i in xrange(len(tuple(tree))):
    node.append([])

#print node
for i in xrange(len(tuple(tree))):
    for j in xrange(len(tuple(tree[0]))):
        entry =  tuple(tree[i])[j]
        if entry < 0:
#           print 'node idex = ', (-entry - 1), node[(-entry - 1)]
            node[i].extend(node[(-entry - 1)])
        else:
            node[i].append(labels[entry])

for i in xrange(len(tuple(tree))):
    print linkdist[i],len(node[i])
    for j in xrange(len(node[i])):
        print 'Cluster ',i+1, node[i][j]

writeclusterfiles('out', datarray, labels, expid, mask=None, geneclusters=tree,
genelinkdist=linkdist, expclusters=None, explinkdist=None, genename=None,
gweight=None, eweight=eweight, uniqid='UNIQID')

